package dev.shadowsoffire.apotheosis.adventure;

import net.minecraft.sounds.SoundEvent;
import net.minecraft.world.item.ItemStack;
import net.minecraft.network.chat.Component;
import net.minecraft.world.item.CreativeModeTab;
import dev.shadowsoffire.placebo.menu.MenuUtil;
import dev.shadowsoffire.apotheosis.adventure.affix.augmenting.AugmentingMenu;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.cutting.GemCuttingMenu;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingMenu;
import dev.shadowsoffire.apotheosis.adventure.affix.reforging.ReforgingMenu;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.core.registries.Registries;
import java.util.function.Supplier;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.gen.ItemFrameGemsProcessor;
import net.minecraft.world.level.levelgen.structure.templatesystem.StructureProcessorType;
import dev.shadowsoffire.apotheosis.adventure.gen.RogueSpawnerFeature;
import dev.shadowsoffire.apotheosis.adventure.gen.BossDungeonFeature2;
import dev.shadowsoffire.apotheosis.adventure.gen.BossDungeonFeature;
import dev.shadowsoffire.apotheosis.util.TooltipItem;
import dev.shadowsoffire.apotheosis.adventure.boss.BossSummonerItem;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.Rarity;
import net.minecraft.world.level.block.Block;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemItem;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvageItem;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.block.SoundType;
import net.minecraft.world.level.block.state.BlockBehaviour;
import dev.shadowsoffire.apotheosis.adventure.affix.augmenting.AugmentingTableBlock;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.cutting.GemCuttingBlock;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingTableBlock;
import dev.shadowsoffire.apotheosis.adventure.affix.reforging.ReforgingTableBlock;
import dev.shadowsoffire.apotheosis.adventure.boss.BossSpawnerBlock;
import net.minecraftforge.registries.RegistryObject;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.placebo.registry.DeferredHelper;

public class Adventure
{
    private static final DeferredHelper R;
    
    public static void bootstrap() {
        Blocks.bootstrap();
        Items.bootstrap();
        Features.bootstrap();
        Menus.bootstrap();
        Tabs.bootstrap();
        Sounds.bootstrap();
    }
    
    static {
        R = (DeferredHelper)Apotheosis.ModularDeferredHelper.create(() -> Apotheosis.enableAdventure);
    }
    
    public static class Blocks
    {
        public static final RegistryObject<BossSpawnerBlock> BOSS_SPAWNER;
        public static final RegistryObject<ReforgingTableBlock> SIMPLE_REFORGING_TABLE;
        public static final RegistryObject<ReforgingTableBlock> REFORGING_TABLE;
        public static final RegistryObject<SalvagingTableBlock> SALVAGING_TABLE;
        public static final RegistryObject<GemCuttingBlock> GEM_CUTTING_TABLE;
        public static final RegistryObject<AugmentingTableBlock> AUGMENTING_TABLE;
        
        private static void bootstrap() {
        }
        
        static {
            BOSS_SPAWNER = Adventure.R.block("boss_spawner", () -> new BossSpawnerBlock(BlockBehaviour.Properties.m_284310_().m_60999_().m_60913_(-1.0f, 3600000.0f).m_222994_()));
            SIMPLE_REFORGING_TABLE = Adventure.R.block("simple_reforging_table", () -> new ReforgingTableBlock(BlockBehaviour.Properties.m_284310_().m_60999_().m_60913_(2.0f, 20.0f), 2));
            REFORGING_TABLE = Adventure.R.block("reforging_table", () -> new ReforgingTableBlock(BlockBehaviour.Properties.m_284310_().m_60999_().m_60913_(4.0f, 1000.0f), 4));
            SALVAGING_TABLE = Adventure.R.block("salvaging_table", () -> new SalvagingTableBlock(BlockBehaviour.Properties.m_284310_().m_60918_(SoundType.f_56736_).m_60978_(2.5f)));
            GEM_CUTTING_TABLE = Adventure.R.block("gem_cutting_table", () -> new GemCuttingBlock(BlockBehaviour.Properties.m_284310_().m_60918_(SoundType.f_56736_).m_60978_(2.5f)));
            AUGMENTING_TABLE = Adventure.R.block("augmenting_table", () -> new AugmentingTableBlock(BlockBehaviour.Properties.m_284310_().m_60999_().m_60913_(4.0f, 1000.0f)));
        }
    }
    
    public static class Items
    {
        public static final RegistryObject<Item> COMMON_MATERIAL;
        public static final RegistryObject<Item> UNCOMMON_MATERIAL;
        public static final RegistryObject<Item> RARE_MATERIAL;
        public static final RegistryObject<Item> EPIC_MATERIAL;
        public static final RegistryObject<Item> MYTHIC_MATERIAL;
        public static final RegistryObject<Item> ANCIENT_MATERIAL;
        public static final RegistryObject<Item> GEM_DUST;
        public static final RegistryObject<Item> GEM_FUSED_SLATE;
        public static final RegistryObject<Item> SIGIL_OF_SOCKETING;
        public static final RegistryObject<Item> SIGIL_OF_WITHDRAWAL;
        public static final RegistryObject<Item> SIGIL_OF_REBIRTH;
        public static final RegistryObject<Item> SIGIL_OF_ENHANCEMENT;
        public static final RegistryObject<Item> SIGIL_OF_UNNAMING;
        public static final RegistryObject<Item> BOSS_SUMMONER;
        public static final RegistryObject<Item> SIMPLE_REFORGING_TABLE;
        public static final RegistryObject<Item> REFORGING_TABLE;
        public static final RegistryObject<Item> SALVAGING_TABLE;
        public static final RegistryObject<Item> GEM_CUTTING_TABLE;
        public static final RegistryObject<Item> AUGMENTING_TABLE;
        public static final RegistryObject<Item> GEM;
        
        private static RegistryObject<Item> rarityMat(final String id) {
            return (RegistryObject<Item>)Adventure.R.item(id + "_material", () -> {
                new SalvageItem(RarityRegistry.INSTANCE.holder(Apotheosis.loc(id)), new Item.Properties());
                return;
            });
        }
        
        private static void bootstrap() {
        }
        
        static {
            COMMON_MATERIAL = rarityMat("common");
            UNCOMMON_MATERIAL = rarityMat("uncommon");
            RARE_MATERIAL = rarityMat("rare");
            EPIC_MATERIAL = rarityMat("epic");
            MYTHIC_MATERIAL = rarityMat("mythic");
            ANCIENT_MATERIAL = rarityMat("ancient");
            GEM_DUST = Adventure.R.item("gem_dust", () -> {
                new Item(new Item.Properties());
                return;
            });
            GEM_FUSED_SLATE = Adventure.R.item("gem_fused_slate", () -> {
                new Item(new Item.Properties());
                return;
            });
            SIGIL_OF_SOCKETING = Adventure.R.item("sigil_of_socketing", () -> {
                new TooltipItem(new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
            SIGIL_OF_WITHDRAWAL = Adventure.R.item("sigil_of_withdrawal", () -> {
                new TooltipItem(new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
            SIGIL_OF_REBIRTH = Adventure.R.item("sigil_of_rebirth", () -> {
                new TooltipItem(new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
            SIGIL_OF_ENHANCEMENT = Adventure.R.item("sigil_of_enhancement", () -> {
                new TooltipItem(new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
            SIGIL_OF_UNNAMING = Adventure.R.item("sigil_of_unnaming", () -> {
                new TooltipItem(new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
            BOSS_SUMMONER = Adventure.R.item("boss_summoner", () -> {
                new BossSummonerItem(new Item.Properties());
                return;
            });
            SIMPLE_REFORGING_TABLE = Adventure.R.item("simple_reforging_table", () -> {
                new BlockItem((Block)Blocks.SIMPLE_REFORGING_TABLE.get(), new Item.Properties());
                return;
            });
            REFORGING_TABLE = Adventure.R.item("reforging_table", () -> {
                new BlockItem((Block)Blocks.REFORGING_TABLE.get(), new Item.Properties().m_41497_(Rarity.EPIC));
                return;
            });
            SALVAGING_TABLE = Adventure.R.item("salvaging_table", () -> {
                new BlockItem((Block)Blocks.SALVAGING_TABLE.get(), new Item.Properties());
                return;
            });
            GEM_CUTTING_TABLE = Adventure.R.item("gem_cutting_table", () -> {
                new BlockItem((Block)Blocks.GEM_CUTTING_TABLE.get(), new Item.Properties());
                return;
            });
            AUGMENTING_TABLE = Adventure.R.item("augmenting_table", () -> {
                new BlockItem((Block)Blocks.AUGMENTING_TABLE.get(), new Item.Properties().m_41497_(Rarity.UNCOMMON));
                return;
            });
            GEM = Adventure.R.item("gem", () -> {
                new GemItem(new Item.Properties());
                return;
            });
        }
    }
    
    public static class Features
    {
        public static final RegistryObject<BossDungeonFeature> BOSS_DUNGEON;
        public static final RegistryObject<BossDungeonFeature2> BOSS_DUNGEON_2;
        public static final RegistryObject<RogueSpawnerFeature> ROGUE_SPAWNER;
        public static final RegistryObject<StructureProcessorType<ItemFrameGemsProcessor>> ITEM_FRAME_GEMS;
        
        private static void bootstrap() {
        }
        
        static {
            BOSS_DUNGEON = Adventure.R.feature("boss_dungeon", (Supplier)BossDungeonFeature::new);
            BOSS_DUNGEON_2 = Adventure.R.feature("boss_dungeon_2", (Supplier)BossDungeonFeature2::new);
            ROGUE_SPAWNER = Adventure.R.feature("rogue_spawner", (Supplier)RogueSpawnerFeature::new);
            ITEM_FRAME_GEMS = Adventure.R.custom("item_frame_gems", Registries.f_256983_, () -> () -> ItemFrameGemsProcessor.CODEC);
        }
    }
    
    public static class Menus
    {
        public static final RegistryObject<MenuType<ReforgingMenu>> REFORGING;
        public static final RegistryObject<MenuType<SalvagingMenu>> SALVAGE;
        public static final RegistryObject<MenuType<GemCuttingMenu>> GEM_CUTTING;
        public static final RegistryObject<MenuType<AugmentingMenu>> AUGMENTING;
        
        private static void bootstrap() {
        }
        
        static {
            REFORGING = Adventure.R.menu("reforging", () -> MenuUtil.posType(ReforgingMenu::new));
            SALVAGE = Adventure.R.menu("salvage", () -> MenuUtil.posType(SalvagingMenu::new));
            GEM_CUTTING = Adventure.R.menu("gem_cutting", () -> MenuUtil.type(GemCuttingMenu::new));
            AUGMENTING = Adventure.R.menu("augmenting", () -> MenuUtil.posType(AugmentingMenu::new));
        }
    }
    
    public static class Tabs
    {
        public static final RegistryObject<CreativeModeTab> ADVENTURE;
        
        private static void bootstrap() {
        }
        
        static {
            ADVENTURE = Adventure.R.tab("adventure", () -> CreativeModeTab.builder().m_257941_((Component)Component.m_237115_("itemGroup.apotheosis.adventure")).m_257737_(() -> ((Item)Items.GEM.get()).m_7968_()).m_257652_());
        }
    }
    
    public static class Sounds
    {
        public static final RegistryObject<SoundEvent> REFORGE;
        
        private static void bootstrap() {
        }
        
        static {
            REFORGE = Adventure.R.sound("reforge");
        }
    }
}
